/*jslint indent:4,node:true*/
/*global describe,before,it,after*/
(function () {

    'use strict';

    var async  = require("async"),
        commonsTesting = require('./commons-testing'),
        expect = require('expect.js'),
        fs     = require("fs"),
        grunt  = require("grunt"),
        horaa = require("horaa"),
        minify = require("../minify-batch"),
        pack   = require("../pack"),
        path   = require("path");

    describe("pack.js + minify.js", function () {
        var horaaPerl;

        before(function (callback) {
            // mock the perl module
            horaaPerl = horaa("../../../perl");
            horaaPerl.hijack("get", function (getter, options, callback) {
                if (getter === "cartridges" && options === "Site") {
                    callback(null, [
                        "PARENT/CartridgeD",
                        "PARENT/CartridgeE",
                        "PARENT/CartridgeB",
                        "Another_Parent/CartridgeC",
                        "PARENT/CartridgeA"
                    ]);
                } else {
                    callback(null, null);
                }
            });

            callback();
        });

        describe("pack.js", function () {
            describe("LESS + CSS", function () {
                var actualPath   = commonsTesting.getPath("webroot-lib"),
                    expectedPath = commonsTesting.getPath("expected");

                it("should create and write the correct content into package-bo.less", function (callback) {
                    pack.packLess({
                        "targetPath": actualPath,
                        "storeType": "Site",
                        "profile": "bo",
                        "fileExtension": "less"
                    }, function (error) {
                        // no error should be reported
                        expect(error).to.not.be.ok();

                        async.parallel([
                            function (callback) {
                                // package-bo.less should exist
                                fs.exists(path.join(actualPath, "package-bo.less"), function (exists) {
                                    expect(exists).to.be.ok();
                                    callback();
                                });
                            },

                            function (callback) {
                                // contents of actual package-bo.less should match content of expected package-bo.less
                                async.parallel({
                                    expected: function (callback) {
                                        fs.readFile(path.join(expectedPath, "package-bo.less"), callback);
                                    },

                                    actual: function (callback) {
                                        fs.readFile(path.join(actualPath, "package-bo.less"), callback);
                                    }
                                }, function (error, result) {
                                    expect(error).to.not.be.ok();
                                    expect(result.expected.toString()).to.eql(result.actual.toString());

                                    callback();
                                });
                            }
                        ], callback);
                    });
                });
            });

            describe("JS", function () {
                var cartridgePath = commonsTesting.getPath("cartridges"),
                    actualPath    = commonsTesting.getPath("webroot-lib"),
                    expectedPath  = commonsTesting.getPath("expected");

                it("should create and write the correct content into package-bo.js and package-bo.info.js", function (callback) {
                    pack.packJs({
                        "sourcePath": cartridgePath,
                        "targetPath": actualPath,
                        "storeType": "Site",
                        "profile": "bo"
                    }, function (error) {
                        // no error should be reported
                        expect(error).to.not.be.ok();

                        async.parallel([
                            function (callback) {
                                // package-bo.js should exist
                                fs.exists(path.join(actualPath, "package-bo.js"), function (exists) {
                                    expect(exists).to.be.ok();
                                    callback();
                                });
                            },

                            function (callback) {
                                // contents of actual package-bo.js should match content of expected package-bo.js
                                async.parallel({
                                    expected: function (callback) {
                                        fs.readFile(path.join(expectedPath, "package-bo.js"), callback);
                                    },

                                    actual: function (callback) {
                                        fs.readFile(path.join(actualPath, "package-bo.js"), callback);
                                    }
                                }, function (error, result) {
                                    expect(error).to.not.be.ok();
                                    expect(result.expected.toString()).to.eql(result.actual.toString());

                                    callback();
                                });
                            },

                            function (callback) {
                                // contents of actual package-bo.info.js should match content of expected
                                // package-bo.info.js
                                async.parallel({
                                    expected: function (callback) {
                                        fs.readFile(path.join(expectedPath, "package-bo.info.js"), callback);
                                    },

                                    actual: function (callback) {
                                        fs.readFile(path.join(actualPath, "package-bo.info.js"), callback);
                                    }
                                }, function (error, result) {
                                    expect(error).to.not.be.ok();
                                    expect(result.expected.toString()).to.eql(result.actual.toString());

                                    callback();
                                });
                            }
                        ], callback);
                    });
                });
            });
        });

        describe("minify.js", function () {
            it("should minify the files and resolve @imports only in the ^package-* files", function (callback) {
                var webrootLib    = commonsTesting.getPath("webroot-lib"),
                    webrootLibMin = commonsTesting.getPath("webroot-lib-min"),
                    expectedPath  = commonsTesting.getPath("expected");

                async.series([
                    function (callback) {
                        minify(webrootLib, webrootLibMin, {}, callback);
                    },

                    function (callback) {
                        async.parallel([
                            function (callback) {
                                // contents of actual a2.js.map should match content of expected a2.js.map
                                async.parallel({
                                    expected: function (callback) {
                                        fs.readFile(path.join(expectedPath, "parent-a2.min.js.map"), callback);
                                    },

                                    actual: function (callback) {
                                        fs.readFile(path.join(webrootLibMin, "parent", "a2.js.map"), callback);
                                    }
                                }, function (error, result) {
                                    expect(error).to.not.be.ok();
                                    expect(result.expected.toString()).to.be(result.actual.toString());

                                    callback();
                                });
                            },

                            function (callback) {
                                // contents of actual minifed package-bo.css should match content of
                                // expected package-bo.css
                                async.parallel({
                                    expected: function (callback) {
                                        fs.readFile(path.join(expectedPath, "package-bo.min.css"), callback);
                                    },

                                    actual: function (callback) {
                                        fs.readFile(path.join(webrootLibMin, "package-bo.css"), callback);
                                    }
                                }, function (error, result) {
                                    expect(error).to.not.be.ok();
                                    expect(result.expected.toString()).to.be(result.actual.toString());

                                    callback();
                                });
                            },

                            function (callback) {
                                // Expect template files not to be in the minfied directory
                                fs.exists(path.join(webrootLibMin, "backbone-someview.tmpl.html"), function (exists) {
                                    expect(exists).to.not.be.ok();
                                    callback();
                                });
                            }
                        ], function () {
                            callback();
                        });
                    }
                ], callback);
            });
        });

        after(function (callback) {
            // clean produced files
            async.parallel([
                function (callback) {
                    var pending = [];
                    grunt.file.expand(path.join(commonsTesting.getPath("webroot-lib"), "package-*")).forEach(function (p) {
                        pending.push(function (callback) {
                            fs.unlink(p, callback);
                        });
                    });

                    async.parallel(pending, callback);
                },

                function (callback) {
                    var pending = [];
                    grunt.file.expand(path.join(commonsTesting.getPath("webroot-lib"), "**", "*.tmpl.js")).forEach(function (p) {
                        pending.push(function (callback) {
                            fs.unlink(p, callback);
                        });
                    });

                    async.parallel(pending, callback);
                },

                function (callback) {
                    fs.exists(commonsTesting.getPath("webroot-lib-min"), function (e) {
                        if (e) {
                            grunt.file["delete"](commonsTesting.getPath("webroot-lib-min"));
                        }

                        callback();
                    });
                },

                function (callback) {
                    // restore perl module
                    horaaPerl.restore("get");

                    callback();
                }
            ], callback);
        });
    });

}());
