/*jslint node:true,nomen:true*/
/*global describe,before,it,after*/
(function () {

    'use strict';

    var expect = require('expect.js'),
        horaa  = require("horaa"),
        path = require("path"),
        commons = require("../commons");

    describe("perl.js", function () {
        var epagesPerlOriginal,
            epagesVersionOriginal,
            epagesWebrootOriginal,
            horaaChildProcess,
            perl,
            perlLocation = path.normalize(path.join(__dirname, "..", "perl")),
            lastPerlCmd;

        before(function (callback) {
            // define evns
            epagesPerlOriginal = process.env.EPAGES_PERL;
            process.env.EPAGES_PERL = "__PERL__";

            // remove perl and child_process module from cache
            require.cache[perlLocation + ".js"] = undefined;
            require.cache.child_process = undefined;

            // mock child_process.exec
            horaaChildProcess = horaa("child_process");
            horaaChildProcess.hijack("exec", function (cmd, callback) {
                var normalCmd = [];

                // Command `__PERL__/bin/perl /home/lorenz/node-epages6/child_process/jsapi.pl -sub "GetCartridges"`
                // Sadly, this only works if none of the paths involved contain any spaces
                // for the `__PERL__` part that is okay, since we mock it anyway,
                // but the other path involves `__dirname`.
                // See fix below.
                cmd = cmd.split(" ");

                // `__dirname` might have spaces too.
                // so we need to put those back together.
                // we expect to have something like
                // `cmd = ["__PERL__/bin/perl", "/home/lorenz/node-epages6/child_process/jsapi.pl","-sub","GetCartridges"]`
                // from now on.
                if (cmd.length > 4) {
                    normalCmd[0] = cmd[0];
                    normalCmd[3] = cmd[cmd.length - 1];
                    normalCmd[2] = cmd[cmd.length - 2];
                    normalCmd[1] = cmd.splice(1, cmd.length - 3).join(" ");
                    cmd = normalCmd;
                }

                lastPerlCmd = cmd;

                switch (cmd[3]) {
                case "\"GetCartridges\"":
                    callback(null, '{"Store":["DE_EPAGES::One", "DE_EPAGES::Two"],"MultiStore":["DE_EPAGES::Three", "DE_EPAGES::Four"],"FlexStore":["DE_EPAGES::Five", "DE_EPAGES::Six"],"FlexSite":["DE_EPAGES::Seven", "DE_EPAGES::Eight"],"MultiStoreSite":[],"Master":["DE_EPAGES::Nine", "DE_EPAGES::Ten"],"Site":["DE_EPAGES::Eleven", "DE_EPAGES::Twelve"],"ASPoolDB":["DE_EPAGES::Thirteen", "DE_EPAGES::Fourteen"],"MultiStoreCommon":["DE_EPAGES::Fiveteen", "DE_EPAGES::Seventeen"],"Common":["DE_EPAGES::Eighteen", "DE_EPAGES::Nineteen"]}');
                    break;
                case "\"GetVersion\"":
                    callback(null, "\"17.2.6\"");
                    break;
                default:
                    callback("This behaviour is not mocked.");
                    break;
                }
            });

            perl = require(perlLocation);

            callback();
        });

        it("should return an error if an unknown getter is requested", function (callback) {
            perl.get("iGuessThisGetterDoesntExist", function (error) {
                expect(error).to.be("Unknown getter iGuessThisGetterDoesntExist");
                callback();
            });
        });

        describe("get cartridges", function () {
            it("should run the correct external command", function (callback) {
                perl.get("cartridges", function () {
                    expect(lastPerlCmd).to.be.an("array");

                    expect(lastPerlCmd[0]).to.be("__PERL__/bin/perl");
                    expect(lastPerlCmd[1]).to.be(
                        commons.fixPath(path.normalize(path.join(__dirname, "..", "child_process", "jsapi.pl")))
                    );
                    expect(lastPerlCmd[2]).to.be("-sub");
                    expect(lastPerlCmd[3]).to.be("\"GetCartridges\"");

                    callback();
                });
            });

            it("should return the cartridges for Store if no store type is given", function (callback) {
                perl.get("cartridges", function (error, result) {
                    expect(error).to.not.be.ok();

                    expect(result).to.be.an("array");

                    expect(result[0]).to.be("DE_EPAGES/One");
                    expect(result[1]).to.be("DE_EPAGES/Two");

                    callback();
                });
            });

            it("should return the cartridges for Site if the store type is set to Site", function (callback) {
                perl.get("cartridges", "Site", function (error, result) {
                    expect(error).to.not.be.ok();

                    expect(result).to.be.an("array");

                    expect(result[0]).to.be("DE_EPAGES/Eleven");
                    expect(result[1]).to.be("DE_EPAGES/Twelve");

                    callback();
                });
            });
        });

        describe("get environment variables", function () {
            before(function (callback) {
                epagesWebrootOriginal = process.env.EPAGES_WEBROOT;
                process.env.EPAGES_WEBROOT = "/this/is/our/web root";

                epagesVersionOriginal = process.env.EPAGES_VERSION;
                process.env.EPAGES_VERSION = "17.2.6";

                callback();
            });

            it("should call the external command for the version", function (callback) {
                perl.get("env", ["webroot"], function () {
                    expect(lastPerlCmd).to.be.an("object");

                    expect(lastPerlCmd[0]).to.be("__PERL__/bin/perl");
                    expect(lastPerlCmd[1]).to.be(
                        commons.fixPath(path.normalize(path.join(__dirname, "..", "child_process", "jsapi.pl")))
                    );
                    expect(lastPerlCmd[2]).to.be("-sub");
                    expect(lastPerlCmd[3]).to.be("\"GetCartridges\"");

                    callback();
                });
            });

            it("should return the webroot", function (callback) {
                perl.get("env", ["webroot"], function (error, result) {
                    expect(error).to.not.be.ok();

                    expect(lastPerlCmd).to.be.an("object");
                    expect(result.webroot).to.be("/this/is/our/web root");

                    callback();
                });
            });

            it("should return EPAGES_VERSION if no variable is given", function (callback) {
                perl.get("env", function (error, result) {
                    expect(error).to.not.be.ok();

                    expect(result).to.be.an("object");
                    expect(result.version).to.be("17.2.6");

                    callback();
                });
            });
        });

        after(function (callback) {
            // restore envs
            process.env.EPAGES_PERL = epagesPerlOriginal;
            process.env.EPAGES_VERSION = epagesVersionOriginal;
            process.env.EPAGES_WEBROOT = epagesWebrootOriginal;

            // restore child process module
            horaaChildProcess.restore("exec");

            callback();
        });
    });

}());
